@@ -1,16 +1,14 @@
-
 from urllib.parse import urlparse
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
-from django.urls import reverse
 from django.test.testcases import TestCase
+from django.urls import reverse
 from django.utils.encoding import force_str
 
 
 class SmartminTestMixin(object):
-
     def fetch_protected(self, url, user, post_data=None, failOnFormValidation=True):
         """
         Fetches the given url. Fails if it can be fetched without first logging in as given user
@@ -20,8 +18,9 @@ class SmartminTestMixin(object):
 
         # can't load if we aren't logged in
         response = self.client.get(url)
-        self.assertRedirect(response, reverse("users.user_login"),
-                            msg="'%s' loaded without being logged in first" % url)
+        self.assertRedirect(
+            response, reverse("users.user_login"), msg="'%s' loaded without being logged in first" % url
+        )
         self.login(user)
 
         # but now we can!
@@ -43,12 +42,12 @@ class SmartminTestMixin(object):
 
     def assertRedirect(self, response, url, status_code=302, msg=None):
         self.assertEqual(response.status_code, status_code, msg=msg)
-        segments = urlparse(response.get('Location', None))
+        segments = urlparse(response.get("Location", None))
         self.assertEqual(segments.path, url, msg=msg)
 
     def assertNotRedirect(self, response, url, msg=None):
         if response.status_code == 302:
-            segments = urlparse(response.get('Location', None))
+            segments = urlparse(response.get("Location", None))
             self.assertNotEqual(segments.path, url, msg=msg)
 
     def create_user(self, username, group_names=()):
@@ -61,12 +60,14 @@ class SmartminTestMixin(object):
         return user
 
     def login(self, user):
-        self.assertTrue(self.client.login(username=user.username, password=user.username),
-                        "Couldn't login as %(user)s / %(user)s" % dict(user=user.username))
+        self.assertTrue(
+            self.client.login(username=user.username, password=user.username),
+            "Couldn't login as %(user)s / %(user)s" % dict(user=user.username),
+        )
 
     def assertNoFormErrors(self, response, post_data=None):
-        if response.status_code == 200 and 'form' in response.context:
-            form = response.context['form']
+        if response.status_code == 200 and "form" in response.context:
+            form = response.context["form"]
 
             if not form.is_valid():
                 errors = []
@@ -91,6 +92,7 @@ class _CRUDLTest(SmartminTest):
     """
     Base class for standard CRUDL test cases
     """
+
     crudl = None
     user = None
     object = None
@@ -132,7 +134,7 @@ class _CRUDLTest(SmartminTest):
             self.login(self.getUser())
 
         # create our object
-        create_page = reverse(self.getCRUDL().url_name_for_action('create'))
+        create_page = reverse(self.getCRUDL().url_name_for_action("create"))
         post_data = self.getCreatePostData()
         self.client.post(create_page, data=post_data)
 
@@ -141,40 +143,40 @@ class _CRUDLTest(SmartminTest):
         return self.object
 
     def testCreate(self):
-        if 'create' not in self.getCRUDL().actions:
+        if "create" not in self.getCRUDL().actions:
             return
-        self._do_test_view('create', post_data=self.getCreatePostData())
+        self._do_test_view("create", post_data=self.getCreatePostData())
 
     def testRead(self):
-        if 'read' not in self.getCRUDL().actions:
+        if "read" not in self.getCRUDL().actions:
             return
-        self._do_test_view('read', self.getTestObject())
+        self._do_test_view("read", self.getTestObject())
 
     def testUpdate(self):
-        if 'update' not in self.getCRUDL().actions:
+        if "update" not in self.getCRUDL().actions:
             return
-        self._do_test_view('update', self.getTestObject(), post_data=self.getUpdatePostData())
+        self._do_test_view("update", self.getTestObject(), post_data=self.getUpdatePostData())
 
     def testDelete(self):
-        if 'delete' not in self.getCRUDL().actions:
+        if "delete" not in self.getCRUDL().actions:
             return
         object = self.getTestObject()
-        self._do_test_view('delete', object, post_data=dict())
+        self._do_test_view("delete", object, post_data=dict())
         self.assertEquals(0, len(self.getManager().filter(pk=object.pk)))
 
     def testList(self):
-        if 'list' not in self.getCRUDL().actions:
+        if "list" not in self.getCRUDL().actions:
             return
         # have at least one object
         self.getTestObject()
-        self._do_test_view('list')
+        self._do_test_view("list")
 
     def testCsv(self):
-        if 'csv' not in self.getCRUDL().actions:
+        if "csv" not in self.getCRUDL().actions:
             return
         # have at least one object
         self.getTestObject()
-        self._do_test_view('csv')
+        self._do_test_view("csv")
 
     def _do_test_view(self, action=None, object=None, post_data=None, query_string=None):
         url_name = self.getCRUDL().url_name_for_action(action)
@@ -194,8 +196,11 @@ class _CRUDLTest(SmartminTest):
 
         view = self.getCRUDL().view_for_action(action)
         if self.getCRUDL().permissions and view.permission is not None:
-            self.assertRedirect(response, reverse("users.user_login"),
-                                msg="Page for '%s' loaded without being logged in first" % action)
+            self.assertRedirect(
+                response,
+                reverse("users.user_login"),
+                msg="Page for '%s' loaded without being logged in first" % action,
+            )
             self.login(self.getUser())
             response = self.client.get(url)
 
@@ -216,8 +221,11 @@ class _CRUDLTest(SmartminTest):
 
     def assertPageGet(self, action, response):
         if response.status_code == 302:
-            self.fail("'%s' resulted in an unexpected redirect to: %s" % (action, response.get('Location')))
-        self.assertEquals(200, response.status_code, )
+            self.fail("'%s' resulted in an unexpected redirect to: %s" % (action, response.get("Location")))
+        self.assertEquals(
+            200,
+            response.status_code,
+        )
 
     def assertPagePost(self, action, response):
         self.assertNoFormErrors(response)
